/*    
       This software may be used and distributed according to the terms of
       the GNU General Public License (GPL), incorporated herein by
       reference.  Drivers based on this skeleton fall under the GPL and
       must retain the authorship (implicit copyright) notice.
 
       This program is distributed in the hope that it will be useful, but
       WITHOUT ANY WARRANTY; without even the implied warranty of
       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
       General Public License for more details.
 */
package cricketdaemon.reader.sim;

import cricketdaemon.clientlib.sim.Coord;
import cricketdaemon.clientlib.sim.SimControl;
import cricketdaemon.Util;
import javax.swing.*;
import java.awt.event.*;
import java.awt.*;

/**
 * Keyboard controls for the simulation GUI.
 *
 * @author Allen Miu
 */
class Controls extends JPanel implements KeyListener {

    JLabel beacon_title = new JLabel("Beacon: ");
    JLabel device_title = new JLabel("Device: ");

    public JLabel x, y, z, orient, curBeaconX, curBeaconY;

    SimControl simCntl;

    public Controls (SimControl s) {
	super();
	simCntl = s;

	setLayout(new FlowLayout());

	//_addInfoLabel("SweepAngle:", "PI/8");
	//_addInfoLabel("SweepDist:", "150");
	//	_addInfoLabel("Beacon ", null);
	//	curBeaconX = _addInfoLabel(" x:", "0");
	//	curBeaconY = _addInfoLabel(" y:", "0");
	_addInfoLabel(" Coords", null);
	x = _addInfoLabel(" x:", "0");
	y = _addInfoLabel(" y:", "0");    
	z = _addInfoLabel(" z:", "0");    
	orient = _addInfoLabel(" angle:", "0.0");    
    }

    private JLabel _addInfoLabel(String title, String value) 
    {
	JLabel t = new JLabel(title, JLabel.LEFT);
	add(t);
	
	if( value != null ) {
	    JLabel v = new JLabel(value, JLabel.RIGHT);
	    add(v);
	    return v;
	}
	return null;
    }
    
    public void keyTyped(KeyEvent e) 
    {
	// do nothing
    }

    public void keyReleased(KeyEvent e)
    {
	// do nothing
    }

    public void keyPressed(KeyEvent e) 
    {
	if(simCntl == null)
	    return;

	int k = KeyEvent.VK_UNDEFINED;
	if(e.isActionKey())
	    k = e.getKeyCode();
	else
	    k = (int) Character.toLowerCase(e.getKeyChar());
	
	//System.out.println("key typed "+k);
	switch(k) {
	case (int) 'f':
	    orient.setText(Util.textOf(Coord.fullCircleInRadians-
				       simCntl.incrDeviceRotate(),3));
	    break;
	case (int) 'd':
	    orient.setText(Util.textOf(Coord.fullCircleInRadians-
				       simCntl.decrDeviceRotate(),3));
	    break;
	case KeyEvent.VK_UP:
	case (int)'i':
	    y.setText(Util.textOf(simCntl.moveDeviceBackward(),3));
	    break;
	case KeyEvent.VK_DOWN:
	case (int)'m':
	    y.setText(Util.textOf(simCntl.moveDeviceForward(),3));
	    break;
	case KeyEvent.VK_LEFT:
	case (int)'j':
	    x.setText(Util.textOf(simCntl.moveDeviceLeft(),3));
	    break;
	case KeyEvent.VK_RIGHT:
	case (int)'k':
	    x.setText(Util.textOf(simCntl.moveDeviceRight(),3));
	    break;
	case (int)'u':
	    z.setText(Util.textOf(simCntl.moveDeviceUp(),3));
	    break;
	case (int)'n':
	    z.setText(Util.textOf(simCntl.moveDeviceDown(),3));
	    break;	
	}
	
	simCntl.notice();
    }
}


